# See end of file for licensing information

package AppLib::CGI::FatalsToBrowser;

# This is a CGI::Carp conversion.

BEGIN { require Carp; }

$main::SIG{__WARN__}=\&AppLib::CGI::FatalsToBrowser::warn;
$main::SIG{__DIE__}=\&AppLib::CGI::FatalsToBrowser::die;
$AppLib::CGI::FatalsToBrowser::VERSION = '1.20';
$AppLib::CGI::FatalsToBrowser::CUSTOM_MSG = undef;

# These are the originals
sub realwarn { CORE::warn(@_); }
sub realdie { CORE::die(@_); }

sub id {
    my $level = shift;
    my($pack, $file, $line, $sub) = caller($level);
    my($id) = $file =~ m|([^/]+)$|;
    return ($file, $line, $id);
}

sub stamp {
    my $time = scalar(localtime);
    my $frame = 0;
    my ($id,$pack,$file);
    do {
	$id = $file;
	($pack,$file) = caller($frame++);
    } until !$file;
    ($id) = $id=~m|([^/]+)$|;
    return "[$time] $id: ";
}

sub warn {
    my $message = shift;
    my($file,$line,$id) = id(1);
    $message .= " at $file line $line.\n" unless $message=~/\n$/;
    _warn($message) if $WARN;
    my $stamp = stamp;
    $message=~s/^/$stamp/gm;
    realwarn $message;
}

sub _warn {
    my $msg = shift;
    if ($EMIT_WARNINGS) {
		# We need to mangle the message a bit to make it a valid HTML
		# comment.  This is done by substituting similar-looking ISO
		# 8859-1 characters for <, > and -.  This is a hack.
		$msg =~ tr/<>-/\253\273\255/;
		chomp $msg;
		print STDOUT "<!-- warning: $msg -->\n";
    } else {
		push @WARNINGS, $msg;
    }
}

sub ineval { _longmess() =~ /eval [\{\']/m }

# The mod_perl package Apache::Registry loads CGI programs by calling
# eval.  These evals don't count when looking at the stack backtrace.
sub _longmess {
    my $message = Carp::longmess();
    my $mod_perl = exists $ENV{MOD_PERL};
    $message =~ s,eval[^\n]+Apache/Registry\.pm.*,,s if $mod_perl;
    return $message;    
}

sub die {
  realdie @_ if ineval;
  my ($message) = @_;
  my $time = scalar(localtime);
  my($file,$line,$id) = id(1);
  $message .= " at $file line $line." unless $message=~/\n$/;
  &fatalsToBrowser($message) if $WRAP and $ENV{SERVER_NAME};
  my $stamp = stamp;
  unless ( $message =~ /^\[/ ) {
  	$message =~ s/^/$stamp/g;
  }
  realdie $message;
}

sub set_message {
    $AppLib::CGI::FatalsToBrowser::CUSTOM_MSG = shift;
    return $AppLib::CGI::FatalsToBrowser::CUSTOM_MSG;
}

sub confess { AppLib::CGI::FatalsToBrowser::die Carp::longmess \@_; }
sub croak   { AppLib::CGI::FatalsToBrowser::die Carp::shortmess \@_; }
sub carp    { AppLib::CGI::FatalsToBrowser::warn Carp::shortmess \@_; }
sub cluck   { AppLib::CGI::FatalsToBrowser::warn Carp::longmess \@_; }

# We have to be ready to accept a filehandle as a reference
# or a string.
sub carpout {
    my($in) = @_;
    my($no) = fileno(to_filehandle($in));
    realdie("Invalid filehandle $in\n") unless defined $no;
    
    open(SAVEERR, ">&STDERR");
    open(STDERR, ">&$no") or 
	( print SAVEERR "Unable to redirect STDERR: $!\n" and exit(1) );
}

sub warningsToBrowser {
    $EMIT_WARNINGS = @_ ? shift : 1;
    _warn(shift @WARNINGS) while $EMIT_WARNINGS and @WARNINGS;
}

# headers
sub fatalsToBrowser {
    my($msg) = @_;
    $msg=~s/&/&amp;/g;
    $msg=~s/>/&gt;/g;
    $msg=~s/</&lt;/g;
    $msg=~s/\n/<br>\n/g;
    $msg=~s/\"/&quot;/g;
    my($wm) = $ENV{SERVER_ADMIN} ? 
	qq[the webmaster (<a href="mailto:$ENV{SERVER_ADMIN}">$ENV{SERVER_ADMIN}</a>)] :
	"this site's webmaster";
    my ($outer_message) = "For help, please send mail to $wm, giving this error message and the time and date of the error.";
    my $mod_perl = exists $ENV{MOD_PERL};
    print STDOUT "Content-type: text/html\n\n" unless $mod_perl;

    warningsToBrowser(1);    # emit warnings before dying

    if ($CUSTOM_MSG) {
	if (ref($CUSTOM_MSG) eq 'CODE') {
	    &$CUSTOM_MSG($msg); # nicer to perl 5.003 users
	    return;
	} else {
	    $outer_message = $CUSTOM_MSG;
	}
    }
    
    my $stacktrace = stacktraceHTML();
    
    my $mess = qq~<H1>Software error:</H1>
	<font face="Verdana, Arial, Helvitica" size="2" color="#990000"><b>$msg</b></font>
	<p>$outer_message</p>
	<p>$stacktrace</p>
	~;


    if ($mod_perl && (my $r = Apache->request)) {
	# If bytes have already been sent, then
	# we print the message out directly.
	# Otherwise we make a custom error
	# handler to produce the doc for us.
	if ($r->bytes_sent) {
	    $r->print($mess);
	    $r->exit;
	} else {
	    $r->status(500);
	    $r->custom_response(500,$mess);
	}
    } else {
	print STDOUT $mess;
    }
}

# Cut and paste from CGI.pm so that we don't have the overhead of
# always loading the entire CGI module.
sub to_filehandle {
    my $thingy = shift;
    return undef unless $thingy;
    return $thingy if UNIVERSAL::isa($thingy,'GLOB');
    return $thingy if UNIVERSAL::isa($thingy,'FileHandle');
    if (!ref($thingy)) {
	my $caller = 1;
	while (my $package = caller($caller++)) {
	    my($tmp) = $thingy=~/[\':]/ ? $thingy : "$package\:\:$thingy"; 
	    return $tmp if defined(fileno($tmp));
	}
    }
    return undef;
}

# Get the stack trace
sub stacktrace {
	my $counter = 0;
	my @callstack = ();
	while( $counter < 500 ) {   # Infinite loop catching
		my @callitem = caller($counter++);
		if ( scalar @callitem ) {
			# Don't show call trace related subs
			next if $callitem[ 3 ] =~ /^AppLib::CGI::FatalsToBrowser::/;

			push @callstack, {
				'package' => $callitem[ 0 ],
				'filename' => $callitem[ 1 ],
				'line' => $callitem[ 2 ],
				'subroutine' => $callitem[ 3 ],
				};
		} else {
			last;
		}
	}
	
	# Return the callstack
	return \@callstack;
}

# Formatted versions
sub stacktraceHTML {
	# Get the stack trace
	my $stack = stacktrace();
	return '' unless @$stack;
	
	# Format and return
	my $content = join "\n", map { "<tr>"
		. "<td bgcolor='#FFFFFF'>$_->{subroutine}</td>"
		. "<td bgcolor='#FFFFFF'>$_->{filename}</td>"
		. "<td bgcolor='#FFFFFF' align='right'>$_->{line}</td>"
		} @$stack;
	
	return qq~
		<table border="1" cellspacing="0" cellpadding="1">
		<tr><th colspan=3>Stack Trace</td></tr>
		<tr><th>Sub</th><th>File</th><th>Line</th></tr>
		$content
		</table>
		~;
}

1;

__END__

# This file is derived from the CPAN module CGI::Carp.

# Copyright 2001-2002, Adam Kennedy.
# Portions Copyright 1995-1998, Lincoln D. Stein. All rights reserved.
#
# This library is free software; you can redistribute it and/or modify
# it under the same terms as Perl itself.
