package CVSMonitor::DataCache;

# This package implements a comprehensive data cache, holding all
# information retrieved from the repository that is not the main
# history.

use strict;
use UNIVERSAL 'isa';
use File::Spec ();
use Class::Autouse qw{File::Flat};





#####################################################################
# Interface Methods

sub exists {
	my $class = shift;	
	my $Version = shift or return undef;
	my $resource = shift or return undef;
	my $filename = $class->_filename( $Version, $resource ) or return undef;

	return -e $filename ? 1 : 0;
}

sub get {
	my $class = shift;
	my $Version = shift or return undef;
	my $resource = shift or return undef;
	my $filename = $class->_filename( $Version, $resource ) or return undef;

	# Make sure the file exists
	return undef unless -e $filename;

	# Slurp in the file
	return File::Flat->slurp( $filename );
}

sub set {
	my $class = shift;
	my $Version = shift or return undef;
	my $resource = shift or return undef;
	my $content = shift or return undef;
	my $filename = $class->_filename( $Version, $resource ) or return undef;

	# Can we write the file
	return undef unless File::Flat->canWrite( $filename );
	
	# Write the file
	return File::Flat->write( $filename, $content ) ? 1 : undef;
}




 	
#####################################################################
# Non-Interface Methods
	
sub _filename {
	my $class = shift;
	my $Version = isa( $_[0], 'CVSMonitor::MetaData::Version' )
		? shift : return undef;
	my $resource = shift or return undef;

	# Build the full file path
	my $filename = $Version->File->filename 
		. '-' . $resource
		. '-' . $Version->revision;
	return File::Spec->catfile( $Version->Module->pathCvsCacheDir, $filename );
}

1;
		
