package CVSMonitor::Graph::Karma;

# This package implements a graph that shows the accumulation of karma over time

use strict;
use UNIVERSAL 'isa';
use base 'CVSMonitor::Graph::Range';
use Class::Autouse qw{
	GD::Graph::lines
	Chart::Math::Axis
	};

# What sort of Range do we need?
sub rangeClass { 'CVSMonitor::Range::Basics' }

# Returns the GD::Image object for the graph
sub gd {
	my $self = shift;
	
	# Build the datasets
	my $dates = $self->{Range}->slice( 'date' ) or return undef;
	my $karma = $self->{Range}->slice( 'karma' ) or return undef;
	my $accum = $self->accumulateSlice( $karma );

	# Add the start data if needed
	if ( $self->getShowStart ) {
		unshift @$dates, 'Start';
		unshift @$karma, 0; unshift @$accum, 0;
	}

	# Create the graph object
	my $graph = GD::Graph::lines->new( $self->getWidth, $self->getHeight ) or return undef;		

	# Set the legend
	$graph->set_legend( 'Karma', 'Total Karma' );
	
	# Since the axis provided by GD::Graph sucks, we will generate our own
	# Use the oh-so-nifty Chart::Math::Axis to acomplish this.
	my $y_max_value = $self->sliceMax( $karma, $accum );
	my $y_min_value = $self->sliceMin( $karma, $accum );
	my $Axis = Chart::Math::Axis->new( $y_max_value, $y_min_value ) or return undef;
	$Axis->include_zero() or return undef;
	if ( $self->{maximum_intervals} ) {
		$Axis->set_maximum_intervals( $self->{maximum_intervals} );
	}
	$Axis->apply_to( $graph );
			
	# Set the calculated graph properties
	$graph->set( 
		title             => $self->getTitle,
		x_labels_vertical => 1,
		y_label           => 'Karma',
		dclrs             => [ qw{green black} ],
		legend_placement  => 'BC',
		zero_axis         => 1,
		skip_undef        => 1,
		);				
	
	# Plot and return
	return $graph->plot( [ $dates, $karma, $accum ] );
}

1;
