package CVSMonitor::Graph::Lines;

# This package implements a Graph that calculate statistics on the quantity
# of lines over time.

use strict;
use UNIVERSAL 'isa';
use base 'CVSMonitor::Graph::Range';
use Class::Autouse qw{
	GD::Graph::lines
	Chart::Math::Axis
	};

# What sort of Range do we need?
sub rangeClass { 'CVSMonitor::Range::Basics' }

# Returns the GD::Image object for the graph
sub gd {
	my $self = shift;
	
	# Build the datasets
	my $dates = $self->{Range}->slice( 'date' ) or return undef;
	my $up = $self->addSlices( 
		$self->{Range}->slice( 'add' ),
		$self->{Range}->slice( 'increase' ),
		) or return undef;
	my $down = $self->addSlices(
		$self->{Range}->slice( 'remove' ),
		$self->{Range}->slice( 'decrease' ),
		) or return undef;
	$down = $self->negateSlice( $down ) or return undef;
	my $size = $self->addSlices( $up, $down ) or return undef;
	my $accum = $self->accumulateSlice( $size );

	# Add the start data if needed
	if ( $self->getShowStart ) {
		unshift @$dates, 'Start';
		unshift @$up, 0; unshift @$down, 0;
		unshift @$size, 0; unshift @$accum, 0;
	}

	# Create the graph object
	my $graph = GD::Graph::lines->new( $self->getWidth, $self->getHeight ) or return undef;		

	# Set the legend
	$graph->set_legend( 'Added', 'Removed', 'Difference', 'Total Size' );
	
	# Since the axis provided by GD::Graph sucks, we will generate our own
	# Use the oh-so-nifty Chart::Math::Axis to acomplish this.
	my $y_max_value = $self->sliceMax( $up, $down, $size, $accum );
	my $y_min_value = $self->sliceMin( $up, $down, $size, $accum );
	my $Axis = Chart::Math::Axis->new( $y_max_value, $y_min_value ) or return undef;
	$Axis->include_zero() or return undef;
	if ( $self->{maximum_intervals} ) {
		$Axis->set_maximum_intervals( $self->{maximum_intervals} );
	}
	$Axis->apply_to( $graph );
			
	# Set the calculated graph properties
	$graph->set( 
		title             => $self->getTitle,
		x_labels_vertical => 1,
		y_label           => 'File Lines',
		dclrs             => [ qw{green red blue black} ],
		legend_placement  => 'BC',
		zero_axis         => 1,
		skip_undef        => 1,
		);				
	
	# Plot and return
	return $graph->plot( [ $dates, $up, $down, $size, $accum ] );
}	

1;
