package CVSMonitor::Graph::Range;

# The CVSMonitor::Graph::Range package provides a graph based on a Range
# for a single Module showing a configurable set of values.

use strict;
use UNIVERSAL 'isa';
use base 'CVSMonitor::Graph';
use Class::Autouse qw{
	GD::Graph::lines
	Chart::Math::Axis
	};





#####################################################################
# Constructor and Static Methods

sub new {
	my $class = shift;
	my $Range = shift or return undef;
	
	# Create and return the object
	my $self = {
		Range             => undef,
		maximum_intervals => 10,
		};
	bless $self, $class;
	
	# Add the standard properties
	$self->_init or return undef;

	# Set the initial Range
	$self->setRange( $Range ) or return undef;

	return $self;
}





#####################################################################
# Properties

# Get and set the Range to use
sub getRange { $_[0]->{Range} }
sub setRange {
	my $self = shift;
	my $Range = shift or return undef;
	my $RangeClass = $self->rangeClass();
	return undef unless isa( $Range, $RangeClass );
	
	# Set the Range
	$self->{Range} = $Range;
	return 1;
}

# Get and set the maximum number of intervals
sub getMaximumIntervals { $_[0]->{maximum_intervals} }
sub setMaximumIntervals { $_[0]->{maximum_intervals} = $_[1] }

# The depth string is generated from the Range
sub depth { ucfirst lc $_[0]->{Range}->depth }
	




#####################################################################
# Graph object generation methods

# Returns the GD::Image object for the graph.
# This is supposed to be done by the subclass.
sub gd {
	return undef;
}





#####################################################################
# Utility methods

# Set a default title for the object
sub defaultTitle {
	my $self = shift;	
	my $depth = $self->depth;
	my $firstDate = $self->{Range}->first->{date};
	my $lastDate = $self->{Range}->last->{date};		
	return "$depth Totals, $firstDate - $lastDate";
}

1;

__END__

# Copyright (C) 2002 Adam Kennedy
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Should you wish to utilise this software under a different licence, 
# please contact the author.
