package CVSMonitor::Statistics;

# This package helps move some large, complex statistics related
# code out into a seperate module, so we don't have to load it all 
# the time.

use strict;
use UNIVERSAL 'isa';

# Generate 'blame' data for a File.
sub fileBlame {
	my $class = shift;
	my $File = shift;
	return undef unless isa( $File, 'CVSMonitor::MetaData::File' );
	
	# Get the most recent HEAD version
	my $Version = $File->getRecentHeadVersion();
	return undef unless $Version;

	return $class->versionBlame( $Version );
}

# Generate blame data for a Version.
# Blame data consists of a list of the Authors that have contributed to a version,
# and how many lines they are responsible for.
# The data is returned as a Hash-of-Hashes, suitable for further processing.
sub versionBlame {
	my $class = shift;
	my $Version = shift;
	return undef unless isa( $Version, 'CVSMonitor::MetaData::Version' );
	
	# Get the Annotation object
	my $Annotation = $Version->Annotation or return undef;
	
	# Turn it into our prefered format
	my $stats = $Annotation->stats or return undef;
	my $authors = $stats->{authors};
	my %blame = ();
	foreach ( keys %$authors ) {
		$blame{$_}->{lines} = $authors->{$_};
	}

	return \%blame;
}

1;
	