package setup::Table;

# Package to generate, large, complex tables.

use strict;
use UNIVERSAL 'isa';
use vars qw{@ISA};
BEGIN { @ISA = 'setup::Error' }






# Generate a complete formatted table.
use vars qw{$defaultcolor};
BEGIN {
	$defaultcolor = {
		bg => '#FFFFFF',
		title => '#CCCCCC',
		total => '#EEEEEE',
		border => '#000000',
		};
}

sub setDefaultColor {
	my ( $class, $name, $value ) = @_;
	return undef unless exists $defaultcolor->{$name};
	$defaultcolor->{$name} = $value;
	return 1;
}		

sub table {
	my ( $class, $data, $opt ) = @_;

	# Set colors
	$opt->{bgcolor} = $defaultcolor->{bg} unless $opt->{bgcolor};
	$opt->{titlecolor} = $defaultcolor->{title} unless $opt->{titlecolor};
	$opt->{totalcolor} = $defaultcolor->{total} unless $opt->{totalcolor};

	# Get the depth and width of the table
	$opt->{rows} = $#$data;
	$opt->{columns} = 0;
	foreach ( map { $#$_ } @$data ) {
		$opt->{columns} = $_ if $_ > $opt->{columns};
	}
		
	# Set the color maps
	my $color = {
		top => $opt->{topcolor} || $opt->{titlecolor},
		left => $opt->{leftcolor} || $opt->{titlecolor},
		right => $opt->{rightcolor} || $opt->{totalcolor},
		bottom => $opt->{bottomcolor} || $opt->{totalcolor},
		normal => $opt->{normalcolor} || $opt->{bgcolor},
		};

	# Set the alignment map
	my $align = {
		top => $opt->{topalign} || 'center',
		left => $opt->{leftalign} || 'left',
		right => $opt->{rightalign} || 'right',
		bottom => $opt->{bottomalign} || 'right',
		normal => $opt->{normalalign} || 'left',
		};

	# Set the bold state
	my $bolded = {
		top => exists $opt->{boldtop} ? $opt->{boldtop} : 1,
		left => exists $opt->{boldleft} ? $opt->{boldleft} : 1,
		right => exists $opt->{boldright} ? $opt->{boldright} : 1,
		bottom => exists $opt->{boldbottom} ? $opt->{boldbottom} : 1,
		normal => exists $opt->{boldnormal} ? $opt->{boldnormal} : 0,
		}; 

	# Set the classes
	my $classes = {
		top => $opt->{topclass},
		left => $opt->{leftclass},
		right => $opt->{rightclass},
		bottom => $opt->{bottomclass},
		normal => $opt->{normalclass},
		};

	# Set the nowrap option
	my $nowrap = $opt->{nowrap} ? ' nowrap' : '';

	# Create the content buffer
	my $content = ();

	# Generate the inside rows/cells
	my $type;
	my $cell;
	foreach my $row ( 0 .. $opt->{rows} ) {
		push @$content, "<tr>";
		foreach my $col ( 0 .. $opt->{columns} ) {
			# Establish the type
			$type = $class->tableCellType( $row, $col, $opt );
			
			# Start the cell content
			$cell = $data->[$row]->[$col];
			$cell = '&nbsp;' if $cell eq '';
			
			# Bold it if it should be bold 
			$cell = "<B>$cell</B>" if $bolded->{$type};
	
			# Class it if it needs to be
			my $class_string = $classes->{$type} ? " class='$classes->{$type}'" : '';
			push @$content, "<td align='$align->{$type}' bgcolor='$color->{$type}'$class_string$nowrap>$cell</td>";
		}
		push @$content, "</tr>";
	}
	
	# Wrap the content in the table skirtings and return
	return $class->tableWrap( $content, $opt );
}
# Wrap a bunch of rows in the double layer funky table
# Returns it as a single string, for convenience

sub tableWrap {
	my $class = shift;
	my $content = shift;
	my $opt = shift || {};
	
	# Set some defaults
	$opt->{cellpadding} = 3 unless exists $opt->{cellpadding};
	$opt->{innerwidth}  = 1 unless exists $opt->{innerwidth};
	$opt->{outerwidth}  = 1 unless exists $opt->{outerwidth};
	$opt->{bordercolor} = $defaultcolor->{border} unless exists $opt->{bordercolor};
	
	# Set the width of the table
	my $width = exists $opt->{width} ? " width='$opt->{width}'" : '';

	return join "\n", (
		"<table border='0' cellspacing='0' cellpadding='$opt->{outerwidth}'$width>",
		"<tr bgcolor='$opt->{bordercolor}'><td>",
		"<table border='0' cellspacing='$opt->{innerwidth}' cellpadding='$opt->{cellpadding}' width='100%'>",
		@$content,
		"</table></td></tr></table>",
		);
}

sub tableCellType {
	my ( $class, $row, $col, $opt ) = @_;
	return 'top' if $row == 0 and $opt->{top};
	return 'left' if $col == 0 and $opt->{left};
	return 'bottom' if $row == $opt->{depth} and $opt->{bottom};
	return 'right' if $col == $opt->{width} and $opt->{right};
	return 'normal';
}

1;
