package setup::Tiny;

# If you thought Config::Simple was small...

require 5.005; # Not tested for 5.004
use strict;
use Fcntl ();

# Set the VERSION
use vars qw{$VERSION};
BEGIN {
	$VERSION = '0.2';
}

# Create the error string
use vars qw{$errstr};
BEGIN { $errstr = '' }

# Create a new, empty, Config object
sub new { bless { }, $_[0] }

sub read {
	$errstr = '';
	my $class = shift;

	# Check the file
	my $file = shift or return $class->_error( 'You did not specify a file name' );
	return $class->_error( "File '$file' does not exist" ) unless -e $file;
	return $class->_error( "'$file' is a directory, not a file" ) unless -f $file;
	return $class->_error( "Insufficient permissions to read '$file'" ) unless -r $file;

	# Create the base object
	my $self = $class->new();
	
	# Open the file
	sysopen( CFG, $file, Fcntl::O_RDONLY() ) 
		or return $class->_error( "Failed to open file '$file': $!" );
	flock( CFG, Fcntl::LOCK_SH() ) 
		or return $class->_error( "Failed to get a read lock on the file '$file'" );
	
	# Get the file's contents
	my @contents = <CFG>;
	
	# Close the file
	flock( CFG, Fcntl::LOCK_UN() )
		or return $class->_error( "Failed to unlock the file '$file'" );	
	close( CFG ) or $class->_error( "Failed to close the file '$file': $!" );
	
	# Parse the file
	my $ns = '_';
	my $counter = 0;
	chomp( @contents );
	foreach ( @contents ) {
		$counter++;
		
		# Skip comments and empty lines
		next if /^\s*(?:\#|\;)/ || /^\s*$/;

		# Handle section headers
		if ( /^\s*\[(.+?)\]\s*/ ) {
			$ns = $1;
			next;
		}
		
		# Handle properties
		if ( /^\s*([^=]+?)\s*=\s*(.*)/ ) {
			$self->{$ns}->{$1} = $2;
			next;
		}
		
		return $self->_error( "Syntax error in '$file' at line $counter: $_" );
	}
	
	return $self;
}

sub write {
	$errstr = '';
	my $self = shift;
	my $file = shift;
	my $mode = shift || 0666;
	unless ( $file ) {
		return $self->_error( 'No file name provided to save to' );
	}

	# Get the contents of the file
	my $contents = $self->write_string();
	
	# Open the file
	sysopen ( CFG, $file, Fcntl::O_WRONLY()|Fcntl::O_CREAT()|Fcntl::O_TRUNC(), $mode )
		or return $self->_error( "Failed to open file '$file' for writing: $!" );
	flock( CFG, Fcntl::LOCK_EX() )
		or return $self->_error( "Failed to get a write lock on the file '$file'" );
	
	print CFG $contents;
	
	# Close the file
	flock( CFG, Fcntl::LOCK_UN() )
		or return $self->_error( "Failed to unlock the file '$file'" );	
	close( CFG ) or $self->_error( "Failed to close the file '$file': $!" );

	return 1;	
}

sub write_string {
	my $self = shift;
	
	my $contents = '';
	foreach my $section ( sort { (($b eq '_') <=> ($a eq '_')) || ($a cmp $b) } keys %$self ) { # Make sure _ is first
		my $block = $self->{$section};
		$contents .= "\n" if length $contents;
		$contents .= "[$section]\n" unless $section eq '_';
		foreach my $property ( sort keys %$block ) {
			$contents .= "$property=$block->{$property}\n";
		}
	}
	
	return $contents;
}
	
# Error handling
sub errstr { $errstr }
sub _error { $errstr = $_[1]; return undef }

1;
